
//------------------------------------------------------------------------------ 
// Copyright (c) 2004 Xilinx, Inc. 
// All Rights Reserved 
//------------------------------------------------------------------------------ 
//   ____  ____ 
//  /   /\/   / 
// /___/  \  /   Vendor: Xilinx 
// \   \   \/    Author: Niall Battson, Xilinx, Inc.
//  \   \        Filename: design_top.vhd
//  /   /        Date Last Modified:  Sept 2, 2004 
// /___/   /\    Date Created: Sept 2, 2004 
// \   \  /  \ 
//  \___\/\___\ 
// 
//
// Revision History: 
//------------------------------------------------------------------------------ 
//   
//   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
//   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
//   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
//   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
//   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
//   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
//   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
//   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
//  WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
//   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
//   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
//   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
//   FOR A PARTICULAR PURPOSE. 
//
//------------------------------------------------------------------------------ 



////////////////////////////////////////////////////////////////////////////////////
//
// Module: MACC FIR FILTER (96 TAP)
//
// Description: Verilog instantiation template for 
// DSP48 embedded MADD blocks arranged as a Systolic
// 16 tap FIR filter.
// 
//
// Device: Virtex-4
//
// Copyright (c) 2000 Xilinx, Inc.  All rights reserved.
//
////////////////////////////////////////////////////////////////////////////////////




module design_top(DIN, CLK, CE, RST, DOUT);
   
	 input [17:0] DIN;
    input CLK;
    input CE;
    input RST;

    output [42:0] DOUT;

    wire [47:0]   DOUT_FULL;
	 wire [17:0]   DATA, COEF;
	 wire          WE_A;
	 wire [6:0]    ADDR_A, ADDR_B;

	 control C1
	 (
	     .CLK(CLK),
		  .CE(CE),
		  .ADDR_A(ADDR_A),
		  .WE_A(WE_A),
		  .ADDR_B(ADDR_B),
		  .RST(RST)
	 );


	dpram MEM1 (
        .addra(ADDR_A),
        .addrb(ADDR_B),
        .clka(CLK),
        .clkb(CLK),
        .dina(DIN),
        .douta(DATA),
        .doutb(COEF),
        .ena(EN),
        .enb(EN),
        .wea(WE_A)
	 );

	 MACC MACC1
	 (
	     .A(DATA),
		  .B(COEF),
		  .CLK(CLK),
		  .LOAD(LOAD),
		  .CE(CE),
		  .RST(RST),
		  .DOUT(DOUT_FULL)
	 );

///////////////////////////////////////////////////////////////////
//
//	  DELAYS FOR LOAD AND CAPTURE SIGNAL
//
///////////////////////////////////////////////////////////////////

	reg  SHIFT_REG0, SHIFT_REG1, SHIFT_REG2, SHIFT_REG3;
	reg  CAPTURE;

   always @(posedge CLK)
	begin
         SHIFT_REG3 <= SHIFT_REG2;
			SHIFT_REG2 <= SHIFT_REG1;
			SHIFT_REG1 <= SHIFT_REG0;
         SHIFT_REG0 <= WE_A;
   end

   assign LOAD = SHIFT_REG3;

	always @(posedge CLK)
	begin
           CAPTURE <= LOAD;
	end

///////////////////////////////////////////////////////////////////
//
//	  CAPTURE REGISTER
//
///////////////////////////////////////////////////////////////////

	reg [42:0] DOUT_WIRE; 

	always @(posedge CLK)
	begin
       if(CAPTURE)
		     DOUT_WIRE <= DOUT_FULL[42:0];
	end

	assign DOUT = DOUT_WIRE;

endmodule
