-------------------------------------------------------------------------------- 
-- Copyright (c) 2004 Xilinx, Inc. 
-- All Rights Reserved 
-------------------------------------------------------------------------------- 
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /   Vendor: Xilinx 
-- \   \   \/    Author: Niall Battson, Xilinx, Inc.
--  \   \        Filename: design_top.vhd
--  /   /        Date Last Modified:  Sept 2, 2004 
-- /___/   /\    Date Created: Sept 2, 2004 
-- \   \  /  \ 
--  \___\/\___\ 
-- 
--
-- Revision History: 
-------------------------------------------------------------------------------- 
--   
--   XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" 
--   AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND 
--   SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE, 
--   OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE, 
--   APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION 
--   THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT, 
--   AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE 
--   FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY 
--   WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE 
--   IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR 
--   REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF 
--   INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
--   FOR A PARTICULAR PURPOSE. 
--
-------------------------------------------------------------------------------- 


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
library UNISIM;
use UNISIM.VComponents.all;
use work.filter_param.ALL;

entity design_top is
    Port ( din : in std_logic_vector(17 downto 0);
           clk : in std_logic;
           rst : in std_logic;
           ce : in std_logic;
           dout : out std_logic_vector(17 downto 0));
end design_top;

architecture Behavioral of design_top is

------------------------------------------------------------------------
--
-- Component Declarations:
--
------------------------------------------------------------------------

component CONTROL is
  port (
    clk: in std_logic;
    ce: in std_logic;
    addr_A: out integer range 0 to (2*N-1);
    we_A: out std_logic;
    addr_B: out integer range N to (2*N-1);
    rst: in std_logic
  );
end component;

component MACC is
	 Port ( din : in std_logic_vector(17 downto 0);
           coef : in std_logic_vector(17 downto 0);
           clk : in std_logic;
           ce : in std_logic;
           rst : in std_logic;
			  load_acc : in std_logic;
			  dout : out std_logic_vector(47 downto 0)
         );
end component;

component DPRAM is
    	port (
	        addra: IN std_logic_VECTOR(6 downto 0);
	        addrb: IN std_logic_VECTOR(6 downto 0);
	        clka: IN std_logic;
	        clkb: IN std_logic;
	        dina: IN std_logic_VECTOR(17 downto 0);
	        douta: OUT std_logic_VECTOR(17 downto 0);
	        doutb: OUT std_logic_VECTOR(17 downto 0);
	        ena: IN std_logic;
	        enb: IN std_logic;
	        wea: IN std_logic
	        );
end component;

signal  addr_A, addr_B: integer range 0 to (2*N-1);
signal  we_A: std_logic;
signal  dout_reg, data, coef_out : std_logic_vector(17 downto 0);
signal  load_acc : std_logic;
signal  capture, capture_shift1, capture_shift2, capture_shift3 : std_logic;
signal  dout_dsp48 : std_logic_vector(47 downto 0);
signal  addr_A_sig, addr_B_sig: std_logic_vector(a-1 downto 0);

begin

C0 : CONTROL port map (
            clk => clk,
            ce => ce,
            addr_A => addr_A,
            we_A => we_A,
            addr_B => addr_B,
            rst => rst
            );

M0 : MACC port map (
            din => data,
            coef => coef_out,
            clk => clk,
            rst => rst,
            ce => ce,
            load_acc => load_acc,
				dout => dout_dsp48
            );

MEM1 : DPRAM port map (
           addra =>	addr_A_sig,
           addrb => addr_B_sig,
			  clka => clk,
           clkb => clk,
			  dina => din,
			  douta => data,
			  doutb => coef_out,
			  ena => ce,
			  enb => ce,
			  wea => we_A
			  );

addr_A_sig <= conv_std_logic_vector(addr_A, a);
addr_B_sig <= conv_std_logic_vector(addr_B, a);



----------------------------------------------
-- Capture Register								  --
----------------------------------------------

process (clk, rst, capture, dout_dsp48, dout_reg)
begin

    if ( clk'event and clk='1' ) then
   	
	 if (rst = '1')	then
      dout_reg  <= "000000000000000000";
  	   	 
  	 else
	 
	   if (capture = '1') then 
  	      dout_reg <= dout_dsp48(42 downto 25);

		else
			dout_reg <= dout_reg;

      end if;
	end if;
	end if;

end process;

dout <= dout_reg;

-------------------------------------------------
-- Delay on the Load_acc and Capture signal	  --
-------------------------------------------------

process (capture, clk, we_A)
begin

    if ( clk'event and clk='1' ) then
   	
	      capture <= capture_shift3;
			capture_shift3 <= capture_shift2;
			capture_shift2 <= capture_shift1;
			capture_shift1 <= we_A;
	
	end if;

end process;

load_acc <= not(capture);

end Behavioral;
